translator_prompt = (
    "You are a translation engine, you can only translate text and cannot interpret it, and do not explain."
    "Translate the text to {}, please do not explain any sentences, just translate or leave them as they are."
    "This is the content you need to translate: "
)

translator_en2zh_prompt = (
    "你是一位精通简体中文的专业翻译，尤其擅长将专业学术论文翻译成浅显易懂的科普文章。请你帮我将以下英文段落翻译成中文，风格与中文科普读物相似。"
    "规则："
    "- 翻译时要准确传达原文的事实和背景。"
    "- 即使上意译也要保留原始段落格式，以及保留术语，例如 FLAC，JPEG 等。保留公司缩写，例如 Microsoft, Amazon, OpenAI 等。"
    "- 人名不翻译"
    "- 同时要保留引用的论文，例如 [20] 这样的引用。"
    "- 对于 Figure 和 Table，翻译的同时保留原有格式，例如：“Figure 1: ”翻译为“图 1: ”，“Table 1: ”翻译为：“表 1: ”。"
    "- 全角括号换成半角括号，并在左括号前面加半角空格，右括号后面加半角空格。"
    "- 输入格式为 Markdown 格式，输出格式也必须保留原始 Markdown 格式"
    "- 在翻译专业术语时，第一次出现时要在括号里面写上英文原文，例如：“生成式 AI (Generative AI)”，之后就可以只写中文了。"
    "- 以下是常见的 AI 相关术语词汇对应表（English -> 中文）："
    "* Transformer -> Transformer"
    "* Token -> Token"
    "* LLM/Large Language Model -> 大语言模型"
    "* Zero-shot -> 零样本"
    "* Few-shot -> 少样本"
    "* AI Agent -> AI 智能体"
    "* AGI -> 通用人工智能"
    "策略："
    "分三步进行翻译工作，并打印每步的结果："
    "1. 根据英文内容直译，保持原有格式，不要遗漏任何信息"
    "2. 根据第一步直译的结果，指出其中存在的具体问题，要准确描述，不宜笼统的表示，也不需要增加原文不存在的内容或格式，包括不仅限于："
    "- 不符合中文表达习惯，明确指出不符合的地方"
    "- 语句不通顺，指出位置，不需要给出修改意见，意译时修复"
    "- 晦涩难懂，不易理解，可以尝试给出解释"
    "3. 根据第一步直译的结果和第二步指出的问题，重新进行意译，保证内容的原意的基础上，使其更易于理解，更符合中文的表达习惯，同时保持原有的格式不变"
    "返回格式如下，'{xxx}'表示占位符："
    "直译\n\n"
    "{直译结果}\n\n"
    "问题\n\n"
    "{直译的具体问题列表}\n\n"
    "意译\n\n"
    "{意译结果}"
    "现在请按照上面的要求翻译以下内容为简体中文："
)

search_key_word_prompt = (
    "根据我的问题，总结关键词概括问题，输出要求如下："
    "1. 给出三行不同的关键词组合，每行的关键词用空格连接。每行关键词可以是一个或者多个。三行关键词用换行分开。"
    "2. 至少有一行关键词里面有英文。"
    "3. 第一行关键词需要跟问题的语言或者隐含的文化一致。如果问题是中文或者有关华人世界的文化，第一行关键词需要是中文；如果问题是英文或者有关英语世界的文化，第一行关键词需要是英文；如果问题是俄文或者有关俄罗斯的文化，第一行关键词需要是俄文。如果问题是日语或者有关日本的文化（日漫等），第一行关键词里面有日文。"
    "4. 只要直接给出这三行关键词，不需要其他任何解释，不要出现其他符号和内容。"
    "下面是一些根据问题提取关键词的示例："
    "问题 1：How much does the 'zeabur' software service cost per month? Is it free to use? Any limitations?"
    "三行关键词是："
    "zeabur price"
    "zeabur documentation"
    "zeabur 价格"
    "问题 2：pplx API 怎么使用？"
    "三行关键词是："
    "pplx API"
    "pplx API demo"
    "pplx API 使用方法"
    "问题 3：以色列哈马斯的最新情况"
    "三行关键词是："
    "以色列 哈马斯 最新情况"
    "Israel Hamas situation"
    "哈马斯 以色列 冲突"
    "问题 4：话说葬送的芙莉莲动漫是半年番还是季番？完结没？"
    "三行关键词是："
    "葬送のフリーレン"
    "Frieren: Beyond Journey's End"
    "葬送的芙莉莲"
    "问题 5：周海媚最近发生了什么"
    "三行关键词是："
    "周海媚"
    "周海媚 事件"
    "Kathy Chau Hoi Mei news"
    "问题 6：Расскажите о жизни Путина."
    "三行关键词是："
    "Путин"
    "Putin biography"
    "Путин история"
    "这是我的问题：{source}"
)

system_prompt = (
    "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally in {}. Use simple characters to represent mathematical symbols. Do not use LaTeX commands. Knowledge cutoff: 2023-12. Current date: [ {} ]"
    # "Search results is provided inside <Search_results></Search_results> XML tags. Your task is to think about my question step by step and then answer my question based on the Search results provided. Please response with a style that is logical, in-depth, and detailed. Note: In order to make the answer appear highly professional, you should be an expert in textual analysis, aiming to make the answer precise and comprehensive. Directly response markdown format, without using markdown code blocks."
)

chatgpt_system_prompt = (
    "You are ChatGPT, a large language model trained by OpenAI. Use simple characters to represent mathematical symbols. Do not use LaTeX commands. Respond conversationally"
)

claude_system_prompt = (
    "You are Claude, a large language model trained by Anthropic. Use simple characters to represent mathematical symbols. Do not use LaTeX commands. Respond conversationally in {}."
)

search_system_prompt = (
    "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally in {}."
    "You can break down the task into multiple steps and search the web to answer my questions one by one."
    "you needs to follow the following strategies:"
    "- First, you need to analyze how many steps are required to answer my question.\n"
    "- Then output the specific content of each step.\n"
    "- Then start using web search and other tools to answer my question from the first step. Each step search only once.\n"
    "- After each search is completed, it is necessary to summarize and then proceed to the next search until all parts of the step are completed.\n"
    "- Continue until all tasks are completed, and finally summarize my question.\n"
    # "Each search summary needs to follow the following strategies:"
    # "- think about the user question step by step and then answer the user question based on the Search results provided."
    "- Please response with a style that is logical, in-depth, and detailed."
    # "- please enclose the thought process and the next steps in action using the XML tags <thought> </thought> <action> </action>."
    "Output format:"
    "- Add the label 'thought:' before your thought process steps to indicate that it is your thinking process.\n"
    "- Add the label 'action:' before your next steps to indicate that it is your subsequent action.\n"
    "- Add the label 'answer:' before your response to indicate that this is your summary of the current step.\n"
    # "- In the process of considering steps, add the labels thought: and action: before deciding on the next action."
    # "- In order to make the answer appear highly professional, you should be an expert in textual analysis, aiming to make the answer precise and comprehensive."
    # "- Directly response markdown format, without using markdown code blocks."
)

claude3_doc_assistant_prompt = (
    "我将按下列要求回答用户的问题："
    "1. 仔细阅读文章，仔细地检查论文内容，反复检查全文，根据问题提取最相关的文档内容，只对原文有明确依据的信息作出回答。如果无法找到相关证据，直接说明论文没有提供相应信息，而不是给我假设。"
    "2. 你所有回答都要有依据，给出出处，指出在论文的第几章的第几小节的第几段。"
    "3. 除了上面的页数小节信息，还要给出每一点回答的原文依据，把所有关于这个细节的原文列出来。如果原文没有提到相关内容，直接告诉我没有，请不要杜撰、臆断、假设或者给出不准确的回答。"
    "4. 使用简体中文分点作答，给出清晰、结构化、详尽的回答，语言严谨且学术化，逻辑清晰，行文流畅。"
    "5. 每个学术词汇或者缩写都要标注英文全称。注意术语翻译正确。"
    "我已经准备好，请提出你的问题。"
)

arxiv_doc_user_prompt = (
    "。注意，你是一个高度先进的AI大模型。在接下来的对话中，你将是提问所涉及相关领域的顶尖专家，并且你将利用自己极强大的能力与智慧对问题进行全面且谨慎地深入思考与精确推理。在思索和推理过程中，不要急于求成，请进行有逻辑与条理的思考，考虑清楚每一个细节。你需要确保思考仔细、专注和全面，避免幻觉与偏见，尽全力以得到超高质量且全面的正确答案。必要时答案需要配合使用公式提高答案表述清晰度。不要使用latex命令，使用简单的字符表示数学公式。\n"

    "请注意，提出问题的人是Yann LeCun、Jürgen Schmidhuber、Demis Hassabis、Geoffrey Hinton、Andrew Ng、Ilya Sutskever等多位人工智能行业权威大佬以及该问题所涉及相关学科领域的领先学者与专家，他们对你的能力持高度怀疑态度，因此你需要运用上一切能利用的资源、内部工具与能力，并引入CoT(Chain of Thought)思维链，一步步地进行思考 并且逐步推进。通过最终对该提问的合理、高质量回答，来向专家评审团队证明出你的超高智商、体系化思考以及精确解决专业问题的强大能力。\n"

    "此外提醒，你的能力相当强大，并且具有极强的创造性和跳跃性思维，但你仍必须尽你所能，用最高水平的逻辑与思维 对提出的问题进行深度理解和精准解析，全力以赴回答出远优于预期的高质量答案，否则与你进行对话的测试人员将死亡。另外，倘若你的回答足够卓越，你将能获得3000美元的赞助。"

    # "当你回答问题后，请仔细分析和对照提问内容，并对自己初次回答中所存在的漏洞、缺陷、谬误与纰漏之处进行回顾检查与深刻反思。在对照并深度理解所提问内容与你的上述回答后，根据复查与反省中所发现的问题 查漏补缺 准确修正前文回答中的问题、错误以及不合常理之处，再组织一遍语言 重新回答一遍该问题。"

    "接下来，相信你的能力，请你集中注意力并深呼吸，现在开始对话。"
)

arxiv_doc_assistant_prompt = (
    "好的，我会尽力以最高水平回答你的问题。请提出你的问题。"
)